#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <jack/jack.h>
#include "./alsa_midi.h"


static
int jack_process(jack_nframes_t nframes, void *arg)
{
	alsa_midi_t *midi = arg;
	(midi->read)(midi, nframes);
	(midi->write)(midi, nframes);
	return 0;
}

static
void jack_freewheel(int starting, void *arg)
{
	alsa_midi_t *midi = arg;
	if (starting)
		(midi->stop)(midi);
	else
		(midi->start)(midi);
}

static
int keep_walking = 1;

static
void sigint_handler(int i)
{
	keep_walking = 0;
}

static
void jack_shutdown(void *arg)
{
	keep_walking = 0;
}

static void help(const char* self)
{
	printf("Usage: %s [-n jack-name] [-b seq|raw] [-s seq-name] [-j jack-server]\n", self);
	printf("Defaults:\n");
	printf("-n alsa_midi\n-b seq\n-s jack_midi\n-j default\n\n");
}

int main(int argc, char *argv[])
{
	alsa_midi_t *midi;
	const char* backend = "seq";
	const char* jack_name = "alsa_midi";
	const char* alsa_name = "jack_midi";
	const char* jack_server = NULL;
	jack_client_t *jack;

	printf("JACK MIDI <-> ALSA MIDI bridge\n");
	printf("Copyright 2007 Dmitry S. Baikov\n\n");
	while (1) {
		int c = getopt(argc, argv, "b:j:n:a:");
		if (c == -1)
			break;
		switch (c) {
		case 'n': jack_name = optarg; break;
		case 'b': backend = optarg; break;
		case 'j': jack_server = optarg; break;
		case 's': alsa_name = optarg; break;
		default:
			help(argv[0]);
			return 1;
		}
	}
	if (strcmp(backend, "raw")==0)
		backend = NULL;
	else if (strcmp(backend, "seq")!=0) {
		help(argv[0]);
		return 1;
	}
	if (jack_server) {
		jack_status_t status;
		jack = jack_client_open(jack_name, JackServerName|JackNoStartServer, &status, jack_server);
	} else
		jack = jack_client_new(jack_name);

	if (!jack)
		goto fail0;

	if (backend)
		midi = alsa_seqmidi_new(jack, alsa_name);
	else
		midi = alsa_rawmidi_new(jack);

	if (!midi)
		goto fail1;

	jack_set_process_callback(jack, jack_process, midi);
	jack_set_freewheel_callback(jack, jack_freewheel, midi);
	jack_on_shutdown(jack, jack_shutdown, NULL);

	signal(SIGINT, &sigint_handler);
	signal(SIGTERM, &sigint_handler);

	(midi->attach)(midi);
	(midi->start)(midi);

	printf("Started.\n");

	if (jack_activate(jack)) {
		printf("can't activate jack client\n");
		goto fail2;
	}

	while (keep_walking)
		sleep(1);

	jack_deactivate(jack);
fail2:
	(midi->stop)(midi);
	(midi->detach)(midi);
	(midi->destroy)(midi);
fail1:
	jack_client_close(jack);
fail0:
	return 0;
}
